@@ -33,7 +33,7 @@
 #' There are two options for the bivariate diagnostic: either provide pairwise minimum of marginally
 #' exponentially distributed margins or provide a \code{n} times 2 matrix with the original data, which
 #' is transformed to exponential margins using the empirical distribution function.
-#'
+#' @keywords internal
 #' @references Wadsworth, J.L. (2016). Exploiting Structure of Maximum Likelihood Estimators for Extreme Value Threshold Selection, \emph{Technometrics}, \bold{58}(1), 116-126, \code{http://dx.doi.org/10.1080/00401706.2014.998345}.
 #'
 #' @author Jennifer L. Wadsworth
@@ -47,8 +47,8 @@
 #' \item \code{k}: final number of thresholds used
 #' \item \code{thresh}: threshold selected by the likelihood ratio procedure
 #' \item \code{qthresh}: quantile level of threshold selected by the likelihood ratio procedure
-#' \item \code{cthresh}: vector of candidate thresholds
-#' \item \code{qcthresh}: quantile level of candidate thresholds
+#' \item \code{thresh0}: vector of candidate thresholds
+#' \item \code{qthresh0}: quantile level of candidate thresholds
 #' \item \code{mle.u}: maximum likelihood estimates for the selected threshold
 #' \item \code{model}: model fitted
 #' }
@@ -60,7 +60,7 @@
 #' W.diag(xdat = abs(rnorm(5000)), model = 'nhpp',
 #'        k = 30, q1 = 0, plots = "PS")
 #' W.diag(rexp(1000), model = 'nhpp', k = 20, q1 = 0)
-#' xbvn <- mvrnorm(n = 6000,
+#' xbvn <- rmnorm(n = 6000,
 #'                 mu = rep(0, 2),
 #'                 Sigma = cbind(c(1, 0.7), c(0.7, 1)))
 #' # Transform margins to exponential manually
@@ -74,36 +74,46 @@
 #'        k = 30, q1 = 0)
 #' }
 #' @export
-W.diag <- function(xdat,
-                   model = c("nhpp", "exp", "invexp"),
-                   u = NULL,
-                   k,
-                   q1 = 0,
-                   q2 = 1,
-                   par = NULL,
-                   M = NULL,
-                   nbs = 1000,
-                   alpha = 0.05,
-                   plots = c("LRT", "WN", "PS"),
-                   UseQuantiles = FALSE,
-                   changepar = TRUE,
-                   ...) {
-    stopifnot(is.logical(UseQuantiles),
-              length(UseQuantiles) == 1L,
-              is.logical(changepar),
-              length(changepar) == 1L)
+W.diag <- function(
+  xdat,
+  model = c("nhpp", "exp", "invexp"),
+  u = NULL,
+  k,
+  q1 = 0,
+  q2 = 1,
+  par = NULL,
+  M = NULL,
+  nbs = 1000,
+  alpha = 0.05,
+  plots = c("LRT", "WN", "PS"),
+  UseQuantiles = FALSE,
+  changepar = TRUE,
+  transform = TRUE,
+  ...
+) {
+  stopifnot(
+    is.logical(UseQuantiles),
+    length(UseQuantiles) == 1L,
+    is.logical(changepar),
+    length(changepar) == 1L
+  )
   model <- match.arg(model)
-  if(!is.null(plots)){
-    plots <- match.arg(plots,
-                       choices = c("LRT", "WN", "PS"),
-                       several.ok = TRUE)
+  if (!is.null(plots)) {
+    plots <- match.arg(plots, choices = c("LRT", "WN", "PS"), several.ok = TRUE)
   }
-  if (ncol(as.matrix(xdat)) == 2 &&
-      model %in% c("exp", "invexp")) {
-    xdat <- -log(1 - apply(xdat, 2, function(y) {
-      rank(y, ties.method = "average") / (length(y) + 1)
-    }))
-    xdat <- pmin(xdat[, 1], xdat[, 2])
+  if (transform) {
+    if (
+      ncol(as.matrix(xdat)) == 2 &&
+        model %in% c("exp", "invexp")
+    ) {
+      xdat <- -log(
+        1 -
+          apply(xdat, 2, function(y) {
+            rank(y, ties.method = "average") / (length(y) + 1)
+          })
+      )
+      xdat <- as.numeric(pmin(xdat[, 1], xdat[, 2]))
+    }
   }
   if (ncol(as.matrix(xdat)) != 1) {
     stop("Invalid input for \"xdat\"")
@@ -158,19 +168,21 @@ W.diag <- function(xdat,
 
 
 .NHPP.diag <-
-  function(xdat,
-           u = NULL,
-           k,
-           q1 = 0,
-           q2 = 1,
-           par = NULL,
-           M = NULL,
-           nbs = 1000,
-           alpha = 0.05,
-           plots = c("LRT", "WN", "PS"),
-           UseQuantiles = TRUE,
-           changepar = changepar,
-           ...) {
+  function(
+    xdat,
+    u = NULL,
+    k,
+    q1 = 0,
+    q2 = 1,
+    par = NULL,
+    M = NULL,
+    nbs = 1000,
+    alpha = 0.05,
+    plots = c("LRT", "WN", "PS"),
+    UseQuantiles = TRUE,
+    changepar = changepar,
+    ...
+  ) {
     unull <- is.null(u)
     if (unull) {
       thresh <- quantile(xdat, q1)
@@ -183,7 +195,7 @@ W.diag <- function(xdat,
     }
     if (is.null(M)) {
       M <- length(xdat[xdat > thresh])
-    }  #why M=nat/3 as default?
+    } #why M=nat/3 as default?
     if (is.null(par)) {
       ppf <-
         fit.pp(
@@ -207,10 +219,14 @@ W.diag <- function(xdat,
     warn <-
       any(eigen(J1$Cov.xi, only.values = TRUE)$val <= .Machine$double.eps)
     if (!unull && warn) {
-      stop("Estimated covariance matrix for xi not positive definite: try different thresholds")
+      stop(
+        "Estimated covariance matrix for xi not positive definite: try different thresholds"
+      )
     }
 
-    while (any(eigen(J1$Cov.xi, only.values = TRUE)$val <= .Machine$double.eps)) {
+    while (
+      any(eigen(J1$Cov.xi, only.values = TRUE)$val <= .Machine$double.eps)
+    ) {
       k <- k - 1
       J1 <-
         .Joint_MLE_NHPP(
@@ -238,114 +254,122 @@ W.diag <- function(xdat,
     wn <-
       .C1(k) %*% J1$mle[, 3] / sqrt(diag(.C1(k) %*% J1$Cov.xi %*% t(.C1(k))))
     nl <- .norm_LRT(x = wn, u = u[-c(1, k + 1)])
-
     nlt <- NULL
     for (j in 1:nbs) {
       nlt[j] <- max(.norm_LRT(x = rnorm(k - 1), u[-c(1, k + 1)])[, 2])
     }
 
-    pval <- length(nlt[nlt > max(nl[, 2])]) / nbs
+    pval <- sum(nlt > max(nl[, 2])) / nbs
 
     if (pval < alpha) {
-      ustar <- nl[nl[, 2] == max(nl[, 2]), 1]
+      # Reject null that the white noise is mean zero throughout
+      # against changepoint alternative
+      # Pick threshold that maximizes the LRT
+      ustar <- nl[which.max(nl[, 2]), 1]
     } else {
+      # No evidence of shift, pick lowest threshold
       ustar <- min(u)
     }
     ind <- u[-(k + 1)] == ustar
-    theta.hat <- J1$mle[ind,]
+    theta.hat <- J1$mle[ind, ]
     if (isTRUE(unull)) {
       qs <- seq(q1, q2, len = k + 1)[-(k + 1)]
       qlthresh <- mean(xdat <= ustar)
-    } else{
+    } else {
       qs <- NULL
       qlthresh <- NULL
     }
-  #   if(!is.null(plots)){
-  #   # Copy graphical elements from ellipsis
-  #   if (changepar) {
-  #     old.par <- par(no.readonly = TRUE)
-  #     on.exit(par(old.par))
-  #     par(mfrow = c(length(plots), 1),
-  #         mar = c(4.5, 4.5, 0.1, 0.1))
-  #   }
-  #   if (is.element("LRT", plots)) {
-  #     if (!UseQuantiles) {
-  #       plot(
-  #         qs,
-  #         c(rep(NA, 2), nl[, 2]),
-  #         xlab = "quantile",
-  #         ylab = "likelihood ratio statistic",
-  #         main = paste("p-value:", pval),
-  #         ...
-  #       )
-  #     } else {
-  #       plot(
-  #         u[-c(k + 1)],
-  #         c(rep(NA, 2), nl[, 2]),
-  #         bty = "l",
-  #         xlab = "threshold",
-  #         ylab = "likelihood ratio statistic",
-  #         main = paste("p-value:", pval),
-  #         ...
-  #       )
-  #     }
-  #   }
-  #
-  #   if (is.element("WN", plots)) {
-  #     if (!UseQuantiles) {
-  #       plot(qs,
-  #            c(NA, wn),
-  #            xlab = "quantile",
-  #            ylab = "white noise",
-  #            bty = "l",
-  #            ...)
-  #       abline(h = 0, col = 2)
-  #       abline(v = mean(xdat <= ustar), col = 4)
-  #     } else {
-  #       plot(u[-c(k + 1)],
-  #            c(NA, wn),
-  #            xlab = "threshold",
-  #            ylab = "white noise",
-  #            bty = "l",
-  #            ...)
-  #       abline(h = 0, col = 2)
-  #       abline(v = ustar, col = 4)
-  #     }
-  #   }
-  #
-  #   if (is.element("PS", plots)) {
-  #     TradCI <-
-  #       cbind(J1$mle[, 3] - qnorm(0.975) * sqrt(diag(J1$Cov.xi)),
-  #             J1$mle[, 3] + qnorm(0.975) * sqrt(diag(J1$Cov.xi)))
-  #     if (!UseQuantiles) {
-  #       plot(
-  #         qs,
-  #         J1$mle[, 3],
-  #         ylim = c(min(TradCI[, 1]), max(TradCI[, 2])),
-  #         xlab = "quantile",
-  #         bty = "l",
-  #         ylab = "shape",
-  #         ...
-  #       )
-  #       lines(qs, TradCI[, 1], lty = 2)
-  #       lines(qs, TradCI[, 2], lty = 2)
-  #       abline(v = mean(xdat <= ustar), col = 4)
-  #     } else {
-  #       plot(
-  #         u[-(k + 1)],
-  #         J1$mle[, 3],
-  #         ylim = c(min(TradCI[, 1]), max(TradCI[, 2])),
-  #         bty = "l",
-  #         xlab = "threshold",
-  #         ylab = "shape",
-  #         ...
-  #       )
-  #       lines(u[-(k + 1)], TradCI[, 1], lty = 2)
-  #       lines(u[-(k + 1)], TradCI[, 2], lty = 2)
-  #       abline(v = ustar, col = 4)
-  #     }
-  #   }
-  # }
+    if (!is.null(plots)) {
+      # Copy graphical elements from ellipsis
+      if (changepar) {
+        old.par <- par(no.readonly = TRUE)
+        on.exit(par(old.par))
+        par(mfrow = c(length(plots), 1), mar = c(4.5, 4.5, 0.1, 0.1))
+      }
+      if (is.element("LRT", plots)) {
+        if (!UseQuantiles) {
+          plot(
+            qs,
+            c(rep(NA, 2), nl[, 2]),
+            xlab = "quantile",
+            ylab = "likelihood ratio statistic",
+            main = paste("p-value:", pval),
+            ...
+          )
+        } else {
+          plot(
+            u[-c(k + 1)],
+            c(rep(NA, 2), nl[, 2]),
+            bty = "l",
+            xlab = "threshold",
+            ylab = "likelihood ratio statistic",
+            main = paste("p-value:", pval),
+            ...
+          )
+        }
+      }
+
+      if (is.element("WN", plots)) {
+        if (!UseQuantiles) {
+          plot(
+            qs,
+            c(NA, wn),
+            xlab = "quantile",
+            ylab = "white noise",
+            bty = "l",
+            ...
+          )
+          abline(h = 0, col = 2)
+          abline(v = mean(xdat <= ustar), col = 4)
+        } else {
+          plot(
+            u[-c(k + 1)],
+            c(NA, wn),
+            xlab = "threshold",
+            ylab = "white noise",
+            bty = "l",
+            ...
+          )
+          abline(h = 0, col = 2)
+          abline(v = ustar, col = 4)
+        }
+      }
+
+      if (is.element("PS", plots)) {
+        TradCI <-
+          cbind(
+            J1$mle[, 3] - qnorm(0.975) * sqrt(diag(J1$Cov.xi)),
+            J1$mle[, 3] + qnorm(0.975) * sqrt(diag(J1$Cov.xi))
+          )
+        if (!UseQuantiles) {
+          plot(
+            qs,
+            J1$mle[, 3],
+            ylim = c(min(TradCI[, 1]), max(TradCI[, 2])),
+            xlab = "quantile",
+            bty = "l",
+            ylab = "shape",
+            ...
+          )
+          lines(qs, TradCI[, 1], lty = 2)
+          lines(qs, TradCI[, 2], lty = 2)
+          abline(v = mean(xdat <= ustar), col = 4)
+        } else {
+          plot(
+            u[-(k + 1)],
+            J1$mle[, 3],
+            ylim = c(min(TradCI[, 1]), max(TradCI[, 2])),
+            bty = "l",
+            xlab = "threshold",
+            ylab = "shape",
+            ...
+          )
+          lines(u[-(k + 1)], TradCI[, 1], lty = 2)
+          lines(u[-(k + 1)], TradCI[, 2], lty = 2)
+          abline(v = ustar, col = 4)
+        }
+      }
+    }
     colnames(J1$mle) <- names(theta.hat) <- c("location", "scale", "shape")
     colnames(J1$Cov.xi) <- rownames(J1$Cov.xi) <- NULL
     ret_list <- list(
@@ -357,19 +381,11 @@ W.diag <- function(xdat,
       k = as.integer(k),
       thresh = as.numeric(ustar),
       qthresh = qlthresh,
-      cthresh = as.numeric(u),
-      qcthresh = qs,
+      thresh0 = as.numeric(u),
+      qthresh0 = qs,
       mle.u = theta.hat,
       model = "nhpp"
     )
-    class(ret_list) <- "mev_thdiag_wadsworth"
-    if(!is.null(plots)){
-      plot(ret_list,
-           plots = plots,
-           changepar = changepar,
-           UseQuantiles = UseQuantiles,
-           ...)
-    }
     invisible(ret_list)
   }
 
@@ -377,18 +393,20 @@ W.diag <- function(xdat,
 #############################################################################################################
 
 .Expl.diag <-
-  function(x,
-           u = NULL,
-           k,
-           q1,
-           q2 = 1,
-           nbs = 1000,
-           alpha = 0.05,
-           plots = c("LRT", "WN", "PS"),
-           UseQuantiles = TRUE,
-           param = "InvRate",
-           changepar = TRUE,
-           ...) {
+  function(
+    x,
+    u = NULL,
+    k,
+    q1,
+    q2 = 1,
+    nbs = 1000,
+    alpha = 0.05,
+    plots = c("LRT", "WN", "PS"),
+    UseQuantiles = TRUE,
+    param = "InvRate",
+    changepar = TRUE,
+    ...
+  ) {
     unull <- is.null(u)
     if (!unull) {
       k <- length(u)
@@ -453,7 +471,7 @@ W.diag <- function(xdat,
     if (isTRUE(unull)) {
       qs <- as.numeric(seq(q1, q2, len = k + 1)[-(k + 1)])
       qlthresh <- mean(x <= ustar)
-    } else{
+    } else {
       qs <- NULL
       qlthresh <- NULL
     }
@@ -568,38 +586,33 @@ W.diag <- function(xdat,
     #   }
     # }
     ret_list <- list(
-        MLE = J1$mle,
-        Cov = J1$Cov,
-        WN = as.numeric(wn),
-        LRT = nl,
-        pval = as.numeric(pval),
-        k = as.integer(k),
-        thresh = as.numeric(ustar),
-        qthresh = qlthresh,
-        cthresh = as.numeric(u),
-        qcthresh = qs,
-        mle.u = as.numeric(theta.hat),
-        model = switch(param,
-                       "InvRate" = "invexp",
-                       "Rate" = "exp")
+      MLE = J1$mle,
+      Cov = J1$Cov,
+      WN = as.numeric(wn),
+      LRT = nl,
+      pval = as.numeric(pval),
+      k = as.integer(k),
+      thresh = as.numeric(ustar),
+      qthresh = qlthresh,
+      thresh0 = as.numeric(u),
+      qthresh0 = qs,
+      mle.u = as.numeric(theta.hat),
+      model = switch(param, "InvRate" = "invexp", "Rate" = "exp")
+    )
+    class(ret_list) <- "mev_thselect_wadsworth"
+    if (!is.null(plots)) {
+      plot(
+        ret_list,
+        plots = plots,
+        changepar = changepar,
+        UseQuantiles = UseQuantiles,
+        ...
       )
-    class(ret_list) <- "mev_thdiag_wadsworth"
-    if(!is.null(plots)){
-      plot(ret_list,
-           plots = plots,
-           changepar = changepar,
-           UseQuantiles = UseQuantiles,
-           ...)
     }
     invisible(ret_list)
   }
 
 
-
-
-
-
-
 #######################################################################################################
 
 #' Joint maximum likelihood estimation for exponential model
@@ -625,12 +638,7 @@ W.diag <- function(xdat,
 #' \item cov joint asymptotic covariance matrix of these MLEs
 #' }
 #' @keywords internal
-.Joint_MLE_Expl <- function(x,
-                            u = NULL,
-                            k,
-                            q1,
-                            q2 = 1,
-                            param) {
+.Joint_MLE_Expl <- function(x, u = NULL, k, q1, q2 = 1, param) {
   if (!is.element(param, c("InvRate", "Rate"))) {
     stop("param should be one of InvRate or Rate")
   }
@@ -655,7 +663,7 @@ W.diag <- function(xdat,
   }
   for (i in 1:k) {
     m[i] <- sum(n[i:k])
-    I[i] <- 1 / thetahat[i] ^ 2
+    I[i] <- 1 / thetahat[i]^2
   }
 
   Tcov <- matrix(0, k, k)
@@ -671,38 +679,31 @@ W.diag <- function(xdat,
 
 #####################################################################################
 
-
-#' Joint maximum likelihood for the non-homogeneous Poisson Process
+# Joint maximum likelihood for the non-homogeneous Poisson Process
 #'
-#' Calculates the MLEs of the parameters (\eqn{\mu}, \eqn{\sigma}, \eqn{\xi}), and joint
-#' asymptotic covariance matrix of these MLEs over a range of thresholds as supplied by the user.
-#' @param x vector of data
-#' @param u optional vector of thresholds. If not supplied, then k thresholds between quantiles (q1, q2) will be used
-#' @param k number of thresholds to consider if \code{u} not supplied
-#' @param q1 lower  quantile to consider for threshold
-#' @param q2 upper quantile to consider for threshold. Default to 1
-#' @param par starting values for the optimization
-#' @param  M  number of superpositions or 'blocks' / 'years' the process corresponds to.
-#' It affects the estimation of \eqn{mu} and \eqn{sigma},
-#' but these can be changed post-hoc to correspond to any number)
-#'
-#' @author Jennifer L. Wadsworth
-#' @return a list with components
-#' \itemize{
-#' \item mle matrix of MLEs above the supplied thresholds; columns are (\eqn{\mu}, \eqn{\sigma}, \eqn{\xi})
-#' \item Cov.all joint asymptotic covariance matrix of all MLEs
-#' \item Cov.mu joint asymptotic covariance matrix of MLEs for \eqn{\mu}
-#' \item Cov.sig joint asymptotic covariance matrix of MLEs for \eqn{\sigma}
-#' \item Cov.xi joint asymptotic covariance matrix of MLEs for \eqn{\xi}
-#' }
-#' @keywords internal
-.Joint_MLE_NHPP <- function(x,
-                            u = NULL,
-                            k,
-                            q1,
-                            q2 = 1,
-                            par,
-                            M) {
+# Calculates the MLEs of the parameters (\eqn{\mu}, \eqn{\sigma}, \eqn{\xi}), and joint
+# asymptotic covariance matrix of these MLEs over a range of thresholds as supplied by the user.
+# @param x vector of data
+# @param u optional vector of thresholds. If not supplied, then k thresholds between quantiles (q1, q2) will be used
+# @param k number of thresholds to consider if \code{u} not supplied
+# @param q1 lower  quantile to consider for threshold
+# @param q2 upper quantile to consider for threshold. Default to 1
+# @param par starting values for the optimization
+# @param  M  number of superpositions or 'blocks' / 'years' the process corresponds to.
+# It affects the estimation of \eqn{mu} and \eqn{sigma},
+# but these can be changed post-hoc to correspond to any number)
+#
+# @author Jennifer L. Wadsworth
+# @return a list with components
+# \itemize{
+# \item mle matrix of MLEs above the supplied thresholds; columns are (\eqn{\mu}, \eqn{\sigma}, \eqn{\xi})
+# \item Cov.all joint asymptotic covariance matrix of all MLEs
+# \item Cov.mu joint asymptotic covariance matrix of MLEs for \eqn{\mu}
+# \item Cov.sig joint asymptotic covariance matrix of MLEs for \eqn{\sigma}
+# \item Cov.xi joint asymptotic covariance matrix of MLEs for \eqn{\xi}
+# }
+# @keywords internal
+.Joint_MLE_NHPP <- function(x, u = NULL, k, q1, q2 = 1, par, M) {
   if (!is.null(u)) {
     k <- length(u)
     x <- x[x > u[1]]
@@ -716,10 +717,8 @@ W.diag <- function(xdat,
   thetahat <- matrix(NA, ncol = 3, nrow = k)
 
   for (i in 1:k) {
-    opt <- fit.pp(xdat = x,
-                  threshold = u[i],
-                  np = M)
-    thetahat[i,] <- opt$estimate
+    opt <- fit.pp(xdat = x, threshold = u[i], np = M)
+    thetahat[i, ] <- opt$estimate
 
     ### Deal with xi <- 0.5
     if (thetahat[i, 3] > -0.5) {
@@ -776,17 +775,25 @@ W.diag <- function(xdat,
 # x - vector of white noise process (WNP, usually normalized estimates of \eqn{xi} or the exponential rate parameter
 # \eqn{1/\eta}) u - vector of thresholds that are associated to the WNP
 
-
 .norm_LRT <- function(x, u) {
   l <- length(u)
   v <-
-    u[-c(1)]  # means two or more obs available for std dev calculation
+    u[-c(1)] # means two or more obs available for std dev calculation
   lr <- NULL
   for (i in 1:length(v)) {
     n1 <- length(x[u <= v[i]])
     num <-
-      .nll_norm(theta = c(mean(x[u <= v[i]]), sd(x[u <= v[i]]) * sqrt((n1 - 1) /
-                                                                        n1)), x = x[u <= v[i]])
+      .nll_norm(
+        theta = c(
+          mean(x[u <= v[i]]),
+          sd(x[u <= v[i]]) *
+            sqrt(
+              (n1 - 1) /
+                n1
+            )
+        ),
+        x = x[u <= v[i]]
+      )
     den <- .nll_norm(theta = c(0, 1), x = x[u <= v[i]])
     lr[i] <- -2 * (num - den)
   }
@@ -794,7 +801,6 @@ W.diag <- function(xdat,
 }
 
 
-
 ###################################################################################
 
 # nll_norm - negative log likelihood for the normal distribution
@@ -803,140 +809,306 @@ W.diag <- function(xdat,
   if (theta[2] < 0) {
     return(1e+11)
   } else {
-    return(-sum(dnorm(
-      x,
-      mean = theta[1],
-      sd = theta[2],
-      log = TRUE
-    )))
+    return(
+      -sum(dnorm(
+        x,
+        mean = theta[1],
+        sd = theta[2],
+        log = TRUE
+      ))
+    )
   }
 }
 
 
-
 ###################################################################################
 
-#' Contrast matrix
-#'
-#' Produces a contrast matrix with (1,-1) elements running down the two diagonals
-#'
-#'@param k number of columns (the number of rows is \code{k-1})
-#'
-#'@return a \code{k-1} x \code{k} contrast matrix
-#'@keywords internal
+# Contrast matrix
+#
+# Produces a contrast matrix with (1,-1) elements running down the two diagonals
+#
+# @param k number of columns (the number of rows is \code{k-1})
+#
+# @return a \code{k-1} x \code{k} contrast matrix
+# @keywords internal
 .C1 <- function(k) {
-  C <- diag(x = 1, nrow = k - 1, ncol = k)
-  C[row(C) + 1 == col(C)] <- -1
-  return(C)
+  Cm <- diag(x = 1, nrow = k - 1, ncol = k)
+  Cm[row(Cm) + 1 == col(Cm)] <- -1
+  return(Cm)
 }
 
+
+#' Sequential analysis diagnostic plots for threshold selection
+#'
 #' @export
-plot.mev_thdiag_wadsworth <-
-  function(x,
-           plots = c("LRT", "WN", "PS"),
-           ...) {
+#' @param x object returned by a call to \code{thselect.wseq}
+#' @param type string giving the plots to produce
+#' @param ... additional arguments passed to plotting function
+#' @return \code{NULL}; the method is used to generate plots
+plot.mev_thselect_wadsworth <-
+  function(x, type = c("wn", "ps"), ...) {
     args <- list(...)
     args$`...` <- NULL #probably not needed anymore
-    if(is.null(args$UseQuantiles)){
-      UseQuantiles <- FALSE
-    } else{
-      UseQuantiles <- args$UseQuantiles
-      args$UseQuantiles <- NULL
-      if(is.null(x$qlevel)){
-        UseQuantiles <- FALSE
-      }
-    }
-    model <- match.arg(arg = x$model,
-                       choices = c("nhpp","exp", "invexp"),
-                       several.ok = FALSE)
-    plots <- match.arg(plots,
-                       choices = c("LRT", "WN", "PS"),
-                       several.ok = TRUE)
-    if(length(plots) < 1){
-      stop("No choice selected; aborting.")
+    model <- match.arg(
+      arg = x$model,
+      choices = c("nhpp", "exp", "invexp"),
+      several.ok = FALSE
+    )
+    plots <- tolower(type)
+    plots <- match.arg(
+      plots,
+      choices = c("wn", "ps"),
+      several.ok = TRUE
+    )
+    if (length(plots) < 1) {
+      stop(
+        "Only options for white noise plot (\"wn\") and parameter stability plot (\"ps\") are supported."
+      )
     }
     # Copy graphical elements from ellipsis
     if (isTRUE(args$changepar)) {
       old.par <- par(no.readonly = TRUE)
       on.exit(par(old.par))
-      par(mfrow = c(length(plots), 1),
-          mar = c(4.5, 4.5, 0.5, 0.5))
+      par(mfrow = c(length(plots), 1), mar = c(4.5, 4.5, 0.5, 0.5))
     }
     args$changepar <- NULL
-    if(!UseQuantiles){
-      xp <- x$cthresh[-c(x$k + 1)]
-      xlab <- "threshold"
-    } else{
-      xp <- x$qthresh
-      xlab <- "quantile"
-    }
+    x$k <- length(x$thresh)
+    xp <- x$thresh[-c(x$k + 1)]
+    xlab <- "threshold"
     args$x <- args$y <- args$xlab <- args$ylab <- NULL
-    if(is.null(args$bty)){
+    if (is.null(args$bty)) {
       args$bty = "l"
     }
-    if (is.element("LRT", plots)) {
-      do.call(what = plot,
-              args = c(list(
-        x = xp,
-        y = c(rep(NA, 2), x$LRT[, 2]),
-        xlab = xlab,
-        ylab = "likelihood ratio"
-              ), args)
+    # if (is.element("lrt", plots)) {
+    #   do.call(
+    #     what = plot,
+    #     args = c(
+    #       list(
+    #         x = xp,
+    #         y = c(rep(NA, 2), x$LRT[, 2]),
+    #         xlab = xlab,
+    #         ylab = "likelihood ratio"
+    #       ),
+    #       args
+    #     )
+    #   )
+    #   mtext(
+    #     cex = 0.8,
+    #     text = paste("p-value:", format.pval(pv = x$pval, eps = 1e-4)),
+    #     side = 3,
+    #     adj = 1
+    #   )
+    # }
+    if (is.element("wn", plots)) {
+      do.call(
+        what = plot,
+        args = c(
+          list(
+            x = xp,
+            y = c(NA, x$whitenoise),
+            xlab = xlab,
+            ylab = "white noise"
+          ),
+          args
+        )
       )
-      mtext(cex = 0.8,
-            text = paste("p-value:", format.pval(pv = x$pval, eps = 1e-4)),
-            side = 3,
-            adj = 1)
-    }
-
-    if (is.element("WN", plots)) {
-      do.call(what = plot,
-              args = c(list(
-                x = xp,
-                 y = c(NA, x$WN),
-                 xlab = xlab,
-                 ylab = "white noise"),
-                args))
       abline(h = 0, col = 2)
-      abline(v = ifelse(UseQuantiles,
-                        x$qthresh,
-                        x$thresh),
-             col = 4)
+      abline(v = x$thresh0, col = 4)
+      mtext(
+        text = paste0("p-value: ", round(x$pval, 3)),
+        side = 1,
+        line = 2,
+        adj = 1
+      )
     }
-    if (is.element("PS", plots)) {
+    if (is.element("ps", plots)) {
       col <- switch(model, nhpp = 3, exp = 1, invexp = 1)
-      TradCI <-
-        cbind(as.matrix(x$MLE)[, col] - qnorm(0.975) * sqrt(diag(x$Cov)),
-              as.matrix(x$MLE)[, col] + qnorm(0.975) * sqrt(diag(x$Cov)))
-      do.call(plot, args = c(list(
-        x = xp,
-        y = as.matrix(x$MLE)[, col],
-        ylim = c(min(TradCI[, 1]), max(TradCI[, 2])),
-        xlab = xlab,
-        ylab = switch(model,
-                      "nhpp" = "shape",
-                      "invexp" = expression(hat(eta)),
-                      "exp" = expression(hat(theta)))),
-        args))
-      lines(xp, TradCI[, 1], lty = 2)
-      lines(xp, TradCI[, 2], lty = 2)
-      abline(v = ifelse(UseQuantiles,
-                        x$qthresh,
-                        x$thresh),
-             col = 4)
+      WaldCI <-
+        cbind(
+          as.matrix(x$coef)[, col] - qnorm(0.975) * sqrt(diag(x$vcov)),
+          as.matrix(x$coef)[, col] + qnorm(0.975) * sqrt(diag(x$vcov))
+        )
+      do.call(
+        plot,
+        args = c(
+          list(
+            x = xp,
+            y = as.matrix(x$coef)[, col],
+            ylim = c(min(WaldCI[, 1]), max(WaldCI[, 2])),
+            xlab = xlab,
+            ylab = switch(
+              model,
+              "nhpp" = "shape",
+              "invexp" = "tail dependence", #expression(hat(eta)),
+              "exp" = "reciprocal tail dependence" #expression(hat(theta))
+            )
+          ),
+          args
+        )
+      )
+      lines(xp, WaldCI[, 1], lty = 2)
+      lines(xp, WaldCI[, 2], lty = 2)
+      abline(v = x$thresh0, col = 4)
     }
     return(invisible(NULL))
   }
 
 
 #' @export
-print.mev_thdiag_wadsworth <-
-  function(x, digits = max(3, getOption("digits") - 3), ...) {
-    cat("Threshold selection method: Wadsworth's white noise test\n based on sequential Poisson process superposition")
-    cat(switch(x$model,
-               "nhpp" = "inhomogeneous Poisson process (shape)",
-               "invexp" = "coefficient of tail dependence \n(exponential, reciprocal rate)",
-               "exp" = "coefficient of tail dependence \n(exponential, rate)"), "\n")
-    cat("Selected threshold:", round(x$thresh, digits), "\n")
+print.mev_thselect_wadsworth <-
+  function(x, digits = min(3, getOption("digits") - 3), ...) {
+    cat(
+      "Threshold selection method: Wadsworth's white noise test\n based on sequential Poisson process superposition.\n"
+    )
+    cat(
+      switch(
+        x$model,
+        "nhpp" = "inhomogeneous Poisson process (shape)",
+        "invexp" = "coefficient of tail dependence (scale)",
+        "exp" = "reciprocal coefficient of tail dependence (rate)"
+      ),
+      "\n"
+    )
+    cat("Selected threshold:", round(x$thresh0, digits), "\n")
     return(invisible(NULL))
   }
+
+
+#' Wadsworth's sequential analysis threshold selection
+#'
+#' Function to produce diagnostic plots and test statistics for the
+#' threshold diagnostics exploiting structure of maximum likelihood estimators
+#' based on the non-homogeneous Poisson process likelihood or the coefficient of tail dependence
+#'
+#' @param xdat a numeric vector or matrix of data to be fitted.
+#' @param thresh vector of candidate thresholds.
+#' @param qlev vector of probabilities for empirical quantiles used in place of the threshold, used if argument \code{thresh} is missing.
+#' @param model string specifying whether the univariate or multivariate diagnostic should be used. Either \code{nhpp} for the univariate model, or \code{exp} (\code{invexp}) for the bivariate exponential model with rate (inverse rate) parametrization. See details.
+#' @param nsim number of Monte Carlo simulations used to assess the null distribution of the test statistic
+#' @param level confidence level of intervals, defaults to 0.95
+#' @param npp number of observations per period for the non-homogeneous point process model. Default to 1.
+#' @param plot logical; if \code{TRUE}, calls the plot routine
+#' @param ... additional parameters passed to internal routine
+#'
+#' @details The function is a wrapper for the univariate (non-homogeneous Poisson process model) and exponential dependence model applied to the minimum component (tail dependence coefficient).
+#' For the latter, the user can select either the rate (\code{"taildep"} or inverse rate parameter  (\code{"rtaildep"}). The inverse rate parametrization  works better for uniformity of the p-value distribution under the likelihood ratio test for the changepoint.
+#'
+#' For the coefficient of tail dependence, users must provide pairwise minimum of marginally exponentially distributed margins (see example)
+#' @references Wadsworth, J.L. (2016). Exploiting Structure of Maximum Likelihood Estimators for Extreme Value Threshold Selection, \emph{Technometrics}, \bold{58}(1), 116-126, \code{http://dx.doi.org/10.1080/00401706.2014.998345}.
+#'
+#' @author Jennifer L. Wadsworth, Léo Belzile
+#' @return an object of class invisible list with components
+#' \itemize{
+#' \item \code{thresh0}: threshold selected by the likelihood ratio procedure
+#' \item \code{thresh}: vector of candidate thresholds
+#' \item \code{coef}: maximum likelihood estimates from all thresholds
+#' \item \code{vcov}: joint asymptotic covariance matrix for shape \eqn{\xi} or coefficient of tail dependence \eqn{\eta}, or it's reciprocal.
+#' \item \code{wn}: values of the white noise process
+#' \item \code{stat}: value of the likelihood ratio test statistic for the changepoint test
+#' \item \code{pval}: \emph{P}-value of the likelihood ratio test
+#' \item \code{mle}: maximum likelihood estimates for the selected threshold
+#' \item \code{model}: model fitted, either \code{nhpp}, \code{exp} or \code{invexp}
+#' \item \code{nsim}: number of Monte Carlo simulations for changepoint test
+#' \item \code{xdat}: vector of observations
+#' }
+#'
+#' @examples
+#' \dontrun{
+#' set.seed(123)
+#' xdat <- abs(rnorm(5000))
+#' thresh <- quantile(xdat, seq(0, 0.9, by = 0.1))
+#' (diag <- thselect.wseq(
+#'  xdat = xdat,
+#'  thresh = thresh,
+#'  plot = TRUE,
+#'  type = "ps"))
+#' # Multivariate example, with coefficient of tail dependence
+#' xbvn <- rmnorm(n = 6000L,
+#'                 mu = rep(0, 2),
+#'                 Sigma = cbind(c(1, 0.7), c(0.7, 1)))
+#' thselect.wseq(
+#'   xdat = xbvn,
+#'   qlev = seq(0, 0.9, length.out = 30),
+#'   model = 'taildep',
+#'   plot = TRUE)
+#' }
+#' @export
+thselect.wseq <- function(
+  xdat,
+  thresh,
+  qlev,
+  model = c("nhpp", "taildep", "rtaildep"),
+  npp = 1,
+  nsim = 1000L,
+  level = 0.95,
+  plot = FALSE,
+  ...
+) {
+  args <- list(...)
+  model <- match.arg(model)
+  mod <- switch(
+    model,
+    rtaildep = "exp",
+    taildep = "invexp",
+    nhpp = "nhpp"
+  )
+  if (mod != "nhpp" && ncol(as.matrix(xdat)) > 1) {
+    xdat <- -log(
+      1 -
+        apply(xdat, 2, function(y) {
+          rank(y, ties.method = "average", na.last = "keep") /
+            (sum(is.finite(y)) + 1)
+        })
+    )
+    xdat <- apply(na.omit(xdat), 1, min)
+    args$transform <- FALSE
+  }
+  if (missing(thresh) && !missing(qlev)) {
+    stopifnot(isTRUE(all(qlev >= 0, qlev < 1, is.finite(qlev))))
+    thresh <- quantile(xdat, probs = qlev)
+  }
+  plots <- args$type
+  if (is.null(plots)) {
+    plots <- c("wn", "ps")
+  }
+  args$type <- NULL
+  wdiag <- do.call(
+    what = "W.diag",
+    args = c(
+      args,
+      list(
+        xdat = xdat,
+        u = thresh,
+        model = mod,
+        nbs = nsim,
+        M = length(xdat) / npp,
+        alpha = 1 - level,
+        plots = NULL
+      )
+    )
+  )
+  if (missing(qlev)) {
+    qlev <- NULL
+  }
+  res <- list(
+    thresh = as.numeric(thresh),
+    thresh0 = wdiag$thresh,
+    qlev = qlev,
+    model = mod,
+    nsim = nsim,
+    level = level,
+    vcov = wdiag$Cov,
+    coef = wdiag$MLE,
+    mle = wdiag$mle.u,
+    whitenoise = wdiag$WN,
+    # P-value is for changepoint, taking the cutoff that maximizes the evidence
+    stat = max(wdiag$LRT[, 2]),
+    pval = wdiag$pval,
+    xdat = xdat
+  )
+  class(res) <- "mev_thselect_wadsworth"
+  if (isTRUE(plot)) {
+    plot(res, type = plots)
+  }
+  return(invisible(res))
+}
