@@ -581,17 +581,21 @@ fit.egp <- function(
       isTRUE(mle$kkt1 & mle$kkt2)
   ) {
     fitted$convergence <- "successful"
-    fitted$vcov <- try(
-      expr = solve(mle$hessian),
+    vcov <- try(
+      expr = suppressWarnings(solve(mle$hessian)),
       silent = TRUE
     )
-    fitted$std.err <- try(
-      expr = sqrt(diag(fitted$vcov)),
-      silent = TRUE
-    )
-    if (
-      inherits(fitted$std.err, what = "try-error") || fitted$param[3] < -0.5
-    ) {
+    if (!inherits(vcov, "try-error")) {
+      fitted$vcov <- vcov
+      se <- try(
+        expr = suppressWarnings(sqrt(diag(fitted$vcov))),
+        silent = TRUE
+      )
+      if (!inherits(se, "try-error")) {
+        fitted$std.err <- se
+      }
+    }
+    if (inherits(vcov, what = "try-error") || fitted$param[3] < -0.5) {
       fitted$vcov <- NULL
       fitted$std.err <- rep(NA, 3)
     }
@@ -707,6 +711,15 @@ egp.fitrange <-
 #' @param type [string] confidence interval type, either \code{wald} or \code{profile}.
 #' @param param [string] parameter, either \code{shape} or additional parameter \code{kappa}
 #' @param plot [logical] if \code{TRUE} (default), return a threshold stability plot
+#' @return an invisible list object of class \code{mev_egp_tstab} with elements
+#' \itemize{
+#' \item{\code{kappa}:}  matrix of parameter estimates and confidence intervals for \eqn{\kappa}, if specified in \code{param}
+#' \item{\code{shape}:}  matrix of parameter estimates and confidence intervals for the shape parameter \eqn{\xi}, if specified in \code{param}
+#' \item{\code{thresh}:} vector of threshold candidates
+#' \item{\code{level}:} level of the confidence intervals
+#' \item{\code{model}:} string giving the EGP model family
+#' \item{\code{type}:} type of confidence interval
+#' }
 #' @export
 #' @examples
 #' xdat <- rgp(n = 1000)
@@ -730,7 +743,7 @@ tstab.egp <- function(
     "logist"
   ),
   param = c("shape", "kappa"),
-  type = c("wald", "profile"),
+  type = c("wald", "lrt"),
   transform = FALSE,
   level = 0.95,
   plot = TRUE,
@@ -778,7 +791,15 @@ tstab.egp <- function(
       egp3 = "pt-power"
     )
   }
-  type <- match.arg(type)
+  type <- match.arg(
+    type,
+    choices = c("lrt", "profile", "wald"),
+    several.ok = TRUE
+  )
+  type <- type[1]
+  if (type == "lrt") {
+    type <- "profile"
+  }
   if (is.numeric(plots)) {
     plots <- as.integer(plots)
   } else {
@@ -794,8 +815,6 @@ tstab.egp <- function(
   if (length(plots) == 0) {
     stop("No option for plots")
   }
-  model <- match.arg(model)
-  type <- match.arg(type)
   if (missing(thresh) && isTRUE(any(c(missing(umin), missing(umax))))) {
     stop(
       "Must provide either minimum and maximum threshold values, or a vector of threshold \"thresh\"."
@@ -1082,9 +1101,10 @@ pegp.G4 <- function(x, kappa, a = 1 / 32) {
   x <- pmin(1, pmax(0, x))
   stopifnot(length(a) == 1L, a > 0, a < 0.5, length(kappa) == 1L, kappa > 0)
   cst <- (pbeta(0.5, kappa, kappa) - pbeta(a, kappa, kappa))
-  (pbeta((0.5 - a) * x + a, shape1 = kappa, shape2 = kappa) -
+  out <- (pbeta((0.5 - a) * x + a, shape1 = kappa, shape2 = kappa) -
     pbeta(a, kappa, kappa)) /
     cst
+  pmin(1, pmax(0, out))
 }
 
 pegp.G5 <- function(x, kappa) {
@@ -1115,7 +1135,8 @@ pegp.G7 <- function(x, kappa) {
     x
   } else {
     cst <- plogis(0, 1, 1 / kappa)
-    (plogis(x, location = 1, scale = 1 / kappa) - cst) / (0.5 - cst)
+    out <- (plogis(x, location = 1, scale = 1 / kappa) - cst) / (0.5 - cst)
+    pmin(1, pmax(0, out))
   }
 }
 
@@ -1251,7 +1272,11 @@ degp.G7 <- function(x, kappa, log = FALSE) {
 
 qegp.G1 <- function(x, kappa, shape) {
   x <- pmin(1, pmax(0, x))
-  1 - (1 - qbeta(x, kappa, 1 / abs(shape)))^(1 / (abs(shape)))
+  if (isTRUE(abs(shape) < 1e-8)) {
+    stop("Not supported")
+  } else {
+    1 - (1 - qbeta(x, kappa, 1 / abs(shape)))^(1 / (abs(shape)))
+  }
 }
 
 qegp.G2 <- function(x, kappa) {
@@ -1356,22 +1381,25 @@ pegp <- function(
       log.p = log.p
     ))
   }
-  p <- pgp(q, loc = 0, scale = scale, shape = shape, lower.tail = lower.tail)
+  p <- pgp(q, loc = 0, scale = scale, shape = shape, lower.tail = TRUE)
   pg <- switch(
     model,
-    "pt-beta" = pegp.G1(pg, kappa = kappa, shape = shape),
-    "pt-gamma" = pegp.G2(pg, kappa = kappa),
-    "pt-power" = pegp.G3(pg, kappa = kappa),
-    "gj-tnorm" = pegp.G5(pg, kappa = kappa),
-    "gj-beta" = pegp.G4(pg, kappa = kappa),
-    "exptilt" = pegp.G6(pg, kappa = kappa),
-    "logist" = pegp.G7(pg, kappa = kappa)
+    "pt-beta" = pegp.G1(p, kappa = kappa, shape = shape),
+    "pt-gamma" = pegp.G2(p, kappa = kappa),
+    "pt-power" = pegp.G3(p, kappa = kappa),
+    "gj-tnorm" = pegp.G5(p, kappa = kappa),
+    "gj-beta" = pegp.G4(p, kappa = kappa),
+    "exptilt" = pegp.G6(p, kappa = kappa),
+    "logist" = pegp.G7(p, kappa = kappa)
   )
-  if (!isTRUE(log.p)) {
-    return(pg)
-  } else {
-    return(log(pg))
+  pg <- pmin(1, pmax(0, pg))
+  if (!isTRUE(lower.tail)) {
+    pg <- 1 - pg
   }
+  if (isTRUE(log.p)) {
+    pg <- log(pg)
+  }
+  return(pg)
 }
 
 #' @rdname egpdist
@@ -1400,6 +1428,14 @@ degp <- function(
     scale > 0
   )
   model <- match.arg(model)
+  if (model %in% c("pt-beta", "pt-gamma") & abs(shape) < 1e-8) {
+    return(dgamma(
+      x = x,
+      scale = scale,
+      shape = kappa,
+      log = log
+    ))
+  }
   pg <- pgp(q = x, scale = scale, shape = shape)
   logdens <- switch(
     model,
@@ -1435,8 +1471,13 @@ qegp <- function(
     "exptilt",
     "logist"
   ),
-  lower.tail = TRUE
+  lower.tail = TRUE,
+  log.p = FALSE
 ) {
+  if (isTRUE(log.p)) {
+    log.p <- FALSE
+    p <- exp(p)
+  }
   stopifnot(
     length(kappa) == 1L,
     length(scale) == 1L,
@@ -1450,7 +1491,7 @@ qegp <- function(
   }
   qu <- rep(NA, length(p))
   vals <- is.finite(p) & p >= 0 & p <= 1
-  if (model %in% c("pt-beta", "pt-gamma") & abs(shape) < 1e-8) {
+  if (model %in% c("pt-beta", "pt-gamma") & (abs(shape) < 1e-8)) {
     qu[vals] <- qgamma(
       p = p[vals],
       scale = scale,
@@ -1459,15 +1500,21 @@ qegp <- function(
     )
   } else {
     qu[vals] <- qgp(
-      switch(
-        model,
-        "pt-beta" = qegp.G1(p[vals], kappa = kappa, shape = shape),
-        "pt-gamma" = qegp.G2(p[vals], kappa = kappa),
-        "pt-power" = qegp.G3(p[vals], kappa = kappa),
-        "gj-tnorm" = qegp.G5(p[vals], kappa = kappa),
-        "gj-beta" = qegp.G4(p[vals], kappa = kappa),
-        "exptilt" = qegp.G6(p[vals], kappa = kappa),
-        "logist" = qegp.G7(p[vals], kappa = kappa)
+      p = pmin(
+        1, # problems with numerical tolerance/rounding to 1e-16 above 1
+        pmax(
+          0,
+          switch(
+            model,
+            "pt-beta" = qegp.G1(p[vals], kappa = kappa, shape = shape),
+            "pt-gamma" = qegp.G2(p[vals], kappa = kappa),
+            "pt-power" = qegp.G3(p[vals], kappa = kappa),
+            "gj-tnorm" = qegp.G5(p[vals], kappa = kappa),
+            "gj-beta" = qegp.G4(p[vals], kappa = kappa),
+            "exptilt" = qegp.G6(p[vals], kappa = kappa),
+            "logist" = qegp.G7(p[vals], kappa = kappa)
+          )
+        )
       ),
       scale = scale,
       shape = shape
@@ -1502,17 +1549,7 @@ regp <- function(
     scale > 0
   )
   model <- match.arg(model)
-  pg <- switch(
-    model,
-    "pt-beta" = qegp.G1(runif(n), kappa = kappa, shape = shape),
-    "pt-gamma" = qegp.G2(runif(n), kappa = kappa),
-    "pt-power" = qegp.G3(runif(n), kappa = kappa),
-    "gj-tnorm" = qegp.G5(runif(n), kappa = kappa),
-    "gj-beta" = qegp.G4(runif(n), kappa = kappa),
-    "exptilt" = qegp.G6(runif(n), kappa = kappa),
-    "logist" = qegp.G7(runif(n), kappa = kappa)
-  )
-  qgp(pg, loc = 0, scale = scale, shape = shape)
+  qegp(p = runif(n), scale = scale, shape = shape, kappa = kappa, model = model)
 }
 
 #' Profile log likelihood for extended generalized Pareto models
@@ -1849,3 +1886,229 @@ egp.pll <- function(
   }
   return(invisible(ans))
 }
+
+
+#' Threshold selection based on extended generalized Pareto models
+#'
+#' Fit an EGP model to data over a range of candidate thresholds \code{thresh} and perform likelihood-based tests of equality for \eqn{\kappa=c}, where \eqn{c=1} for all regular models and $\eqn{c=0} for the \code{'gj-tnorm'} and \code{'logist'} models, for which the generalized Pareto special case corresponds to a value of \eqn{\kappa} occuring on the boundary of the parameter space.
+#'
+#' The threshold selection procedure returns chi-square statistics (\code{stat}) for Wald or profile likelihood ratio tests, along with p-values (\code{pval}) obtained from large sample distribution. The threshold returned is the lowest for which all further higher thresholds fail to reject the null hypothesis of \eqn{\kappa=c}, or equivalently of generalized Pareto tail.
+#' @inheritParams tstab.egp
+#' @param type choice of test statistic, either \code{wald} for Wald-based intervals, or \code{lrt} for profile likelihood ratio test.
+#' @param plot [logical] if \code{TRUE}, return a plot of p-values against threshold
+#' @param ... additional arguments, passed to plotting routine
+#' @return an invisible list of class \code{mev_thselect_egp} with elements
+#' \itemize{
+#' \item{\code{thresh}:} vector of threshold candidates
+#' \item{\code{thresh0}:} selected threshold among candidates
+#' \item{\code{coef}:}  vector of parameter estimates for \eqn{\kappa}
+#' \item{\code{stat}:} squared version of the test statistic
+#' \item{\code{pval}:} p-value obtained from the  \eqn{\chi^2_1} approximation
+#' \item{\code{level}:} level of the confidence intervals
+#' \item{\code{model}:} string giving the EGP model family
+#' \item{\code{type}:} type of confidence interval
+#' }
+#' @export
+#' @examples
+#' ths <- thselect.egp(
+#'   xdat = rexp(1000),
+#'   thresh = qexp(c(0.8,0.9,0.95)),
+#'   model = "pt-power")
+#' print(ths)
+#' plot(ths)
+thselect.egp <- function(
+  xdat,
+  thresh,
+  model = c(
+    "pt-beta",
+    "pt-gamma",
+    "pt-power",
+    "gj-tnorm",
+    "gj-beta",
+    "exptilt",
+    "logist"
+  ),
+  type = c("wald", "lrt"),
+  level = 0.95,
+  transform = FALSE,
+  plot = FALSE,
+  ...
+) {
+  model <- match.arg(model)
+  regular <- !model %in% c("logist", "gj-tnorm")
+  tval <- ifelse(isTRUE(regular), 1, 0)
+  type <- match.arg(type)
+  transform <- isTRUE(transform)
+  if (type == "lrt" | !regular) {
+    transform <- FALSE
+  }
+  if (transform) {
+    tval <- log(tval + 1e-8)
+  }
+  if (length(thresh) <= 1) {
+    stop(
+      "Invalid argument\"thresh\" provided;\n please use a vector of threshold candidates of length at least 2"
+    )
+  }
+  nt <- length(thresh)
+  kappa <- stat <- pval <- rep(NA, length.out = nt)
+  for (i in seq_along(thresh)) {
+    mle <- try(
+      fit.egp(
+        xdat = xdat,
+        thresh = thresh[i],
+        model = model,
+        method = "Nelder"
+      ),
+      silent = TRUE
+    )
+    if (inherits(mle, "try-error")) {
+      next
+    } else {
+      est <- coef(mle)
+      kappa[i] <- est[1]
+      if (type == "wald") {
+        if (transform) {
+          est[1] <- log(est[1] + 1e-8)
+          fn_numderiv <- function(par) {
+            -egp.ll(
+              par = c(exp(par[1]), par[2], par[3]),
+              xdat = xdat,
+              thresh = thresh[i],
+              model = model
+            )
+          }
+          opt <- optim(
+            par = est,
+            fn = fn_numderiv,
+            method = "Nelder",
+            hessian = TRUE
+          )
+          se <- try(
+            expr = suppressWarnings(
+              sqrt(diag(solve(opt$hessian)))[1]
+            ),
+            silent = TRUE
+          )
+          if (inherits(se, "try-error")) {
+            se <- NA
+          }
+        } else {
+          se <- mle$std.err[1]
+        }
+        # Report wald on the chi-square (non-directional stat)
+        stat[i] <- ((est[1] - tval) / se)^2
+        pval[i] <- pchisq(stat[i], df = 1, lower.tail = FALSE)
+        # Boundary case
+        if (isTRUE(all.equal(kappa[i], 0, check.attributes = FALSE))) {
+          pval[i] = 0.5 + 0.5 * pval
+        }
+      } else if (type == "lrt") {
+        alt <- try(
+          fit.egp(
+            xdat = xdat,
+            thresh = thresh[i],
+            fpar = list(kappa = tval),
+            model = model,
+            method = "Nelder"
+          ),
+          silent = TRUE
+        )
+        if (!inherits(alt, "try-error")) {
+          stat[i] <- 2 * (alt$nllh - mle$nllh)
+          pval[i] <- pchisq(stat[i], df = 1, lower.tail = FALSE)
+          if (isTRUE(all.equal(kappa[i], 0, check.attributes = FALSE))) {
+            pval[i] <- 0.5 + 0.5 * pval
+          }
+        }
+      }
+    }
+  }
+  stopifnot(isTRUE(level < 1), isTRUE(level > 0))
+  reject <- which(pval < (1 - level) & !is.na(pval))
+  if (length(reject) == 0L) {
+    thindex <- 1L
+  } else {
+    thindex <- max(reject) + 1L
+  }
+  thresh0 <- thresh[thindex]
+  obj <- list(
+    thresh = as.numeric(thresh),
+    thresh0 = as.numeric(thresh0),
+    coef = kappa,
+    stat = stat,
+    pval = pval,
+    level = level,
+    model = model,
+    type = type
+  )
+
+  class(obj) <- c("mev_egp_thselect")
+  if (isTRUE(plot)) {
+    plot(obj, ...)
+  }
+  return(invisible(obj))
+}
+#' @export
+plot.mev_egp_thselect <- function(x, ...) {
+  args <- list(...)
+  # Produce the plot ......
+  if (is.null(args$ylab)) {
+    args$ylab <- "p-value"
+  }
+  if (is.null(args$bty)) {
+    args$bty <- "l"
+  }
+  if (is.null(args$xlab)) {
+    args$xlab <- "threshold"
+  }
+  if (is.null(args$type)) {
+    args$type <- "b"
+  }
+  if (is.null(args$pch)) {
+    args$pch <- 20
+  }
+  if (is.null(args$ylim)) {
+    args$ylim <- c(0, 1)
+  }
+  args$x <- x$thresh
+  args$y <- x$pval
+  do.call(what = "plot", args = args)
+  mtext(
+    switch(x$type, lrt = "likelihood ratio", wald = "Wald"),
+    side = 1,
+    line = 2,
+    adj = 1,
+    outer = FALSE
+  )
+  abline(h = 1 - x$level, lty = 2)
+  return(invisible(NULL))
+}
+
+#' @export
+print.mev_egp_thselect <- function(
+  x,
+  digits = min(3, getOption("digits") - 3),
+  ...
+) {
+  cat(
+    "Threshold selection method: extended generalized Pareto model.\n"
+  )
+  method <- ifelse(x$type == "wald", "Wald test", "likelihood ratio test")
+  cat(
+    "Largest threshold above which we always fail to reject ",
+    "\n",
+    "the null hypothesis of common generalized Pareto at level",
+    1 - x$level,
+    "\n"
+  )
+  if (!is.na(x$thresh0)) {
+    cat("Selected threshold:", round(x$thresh0, digits), "\n")
+  } else {
+    cat(
+      "No selected threshold: null hypothesis rejected at the highest threshold considered.\n"
+    )
+  }
+
+  return(invisible(NULL))
+}
