@@ -23,14 +23,17 @@
 #' @param B number of bootstrap replications
 #' @param uq logical; if \code{TRUE}, generate bootstrap samples accounting for the sampling distribution of parameters
 #' @param neval number of points at which to estimate the metric. Default to 1000L
-#' @param ci level of symmetric confidence interval. Default to 0.95
+#' @param level level of symmetric confidence interval. Default to 0.95
+#' @param plot logical; if \code{TRUE}, returns a plot
+#' @param ... additional arguments for backward compatibility
 #' @return an invisible list with components
 #' \itemize{
 #' \item \code{thresh}: scalar threshold minimizing criterion
 #' \item \code{thresh0}: vector of candidate thresholds
 #' \item \code{metric}: value of the metric criterion evaluated at each threshold
 #' \item \code{type}: argument \code{type}
-#' \item \code{dist}: argument \code{dist}
+#' \item \code{dist}: argument \code{dist},
+#' \item \code{level}: level of confidence interval, from \code{level}
 #' }
 #' @export
 #' @references Varty, Z. and J.A. Tawn and P.M. Atkinson and S. Bierman (2021+), Inference for extreme earthquake magnitudes accounting for a time-varying measurement process.
@@ -39,7 +42,9 @@
 #' @examples
 #' xdat <- rexp(1000, rate = 1/2)
 #' thresh <- quantile(xdat, prob = c(0.25,0.5, 0.75))
-#'
+#' thv <- thselect.vmetric(xdat, thresh, B = 99)
+#' plot(thv)
+#' print(thv)
 thselect.vmetric <- function(
   xdat,
   thresh,
@@ -48,8 +53,15 @@ thselect.vmetric <- function(
   dist = c("l1", "l2"),
   uq = FALSE,
   neval = 1000L,
-  ci = 0.95
+  level = 0.95,
+  plot = FALSE,
+  ...
 ) {
+  args <- list(...)
+  if (!is.null(args$ci)) {
+    level <- args$ci
+  }
+  ci <- level
   stopifnot(ci > 0, ci < 1, length(ci) == 1L, is.finite(ci))
   dist <- match.arg(dist)
   type <- match.arg(type)
@@ -189,6 +201,7 @@ thselect.vmetric <- function(
     scale = scale,
     shape = shape,
     xdat = xdat,
+    level = level,
     tolerance = t(apply(
       boot_tolerance[[cindex]],
       1,
@@ -197,6 +210,9 @@ thselect.vmetric <- function(
     ))
   )
   class(res) <- "mev_thselect_vmetric"
+  if (isTRUE(plot)) {
+    plot(res)
+  }
   return(invisible(res))
 }
 
@@ -280,6 +296,8 @@ plot.mev_thselect_vmetric <-
     } else {
       thresh <- x$thresh0
       thid <- which(x$thresh == thresh)
+      probs <- sort(probs)[1:2]
+      stopifnot(probs[1] > 0, probs[2] < 1)
       if (x$type == "pp") {
         obs_quant_sim <- switch(
           type,
@@ -408,8 +426,7 @@ plot.mev_thselect_vmetric <-
         col = "grey90",
         border = NA
       )
-      probs <- sort(probs)
-      stopifnot(probs[1] > 0, probs[2] < 1)
+
       segments(
         x0 = xp,
         x1 = xp,
